class Wizard {
	constructor(game){
		this.game = game;
		
		// Rendering
		this.texture = game.textures.players;
		this.map = game.atlas.players.wizard[0];
		
		// Scale
		this.scale = 1;
		this.width =  this.scale * this.map[2];
		this.height = this.scale * this.map[3];
		
		this.x = 330  / 2;
		this.y = 330 / 2;
		
		// Physics
		this.r = 0;
		this.vr = 0;
		this.vx = 0;
		this.vy = 0;
		this.ar = 0;
		
		// Gravity
		this.axG1 = 9;
		this.ayG1 = 9;
		this.vxDampG1 = 1.3;
		this.vyDampG1 = 1.3;
		
		this.axG0 = 2;
		this.ayG0 = 2
		this.vxDampG0 = 1.03;
		this.vyDampG0 = 1.03;
		
		// Wizard
		this.maxLife = 6;
		this.life = this.maxLife;
		
		this.shieldMax = 3;
		this.shield = this.shieldMax;
		this.shieldCooldown = 0;
		this.shieldCooldownMax = 2;
		
		this.spells = [ MageBlade, Bolt, Flamethrower, FireBomb ];
		this.spellCooldowns = [ 0.33, 0.8, 1.7, 2.5 ];
		this.spellCooldown = 0;
		this.spellIndex = 1;
		this.spellCount = 0;
		this.spellCooldownMax = this.spellCooldowns[this.spellIndex];
	}
	
	
	damage(amount){
		if(this.life <= 0) return;
		
		var that = this;
		this.shieldCooldown = this.shieldCooldownMax;
		
		this.game.animations.add(null, function(){
			that.game.context.fillStyle = "#fff";
			that.game.context.globalAlpha = 0.2;
			that.game.context.fillRect(0, 0, that.game.ratio.width/that.game.scale, that.game.ratio.height/that.game.scale);
			that.game.context.globalAlpha = 1;
		}, 0.04);
		
		if(this.shield > 0){
			var that = this;
			this.game.animations.add(function(){ if(that.life>0) that.shield -= amount * that.game.loop.delta / 0.2; }, null, 0.2);
		}
		
		else {
			this.life -= amount;
		}
	}
	
	
	update(delta, input){
		var actionMoveLeft  = input.keys[65]   || input.keys[37],
			actionMoveRight = input.keys[68]   || input.keys[39],
			actionMoveUp    = input.keys[87]   || input.keys[38],
			actionMoveDown  = input.keys[83]   || input.keys[40],
			actionShoot     = input.mouse.down || input.keys[88],
			
			mouseX = (input.mouse.x - this.game.ratio.width /2) / this.game.scale + this.game.sceneX,
			mouseY = (input.mouse.y - this.game.ratio.height/2) / this.game.scale + this.game.sceneY;
		
		// Sort some input stuff
		if(actionMoveLeft && actionMoveRight) actionMoveLeft = actionMoveRight = false;
		if(actionMoveUp   && actionMoveDown ) actionMoveUp   = actionMoveDown  = false;
		
		// Rotation
		this.r = Math.atan2(this.y - mouseY, this.x - mouseX) * (180 / Math.PI) - 90;
		
		// Gravity
		var gravity = this.game.scene.getRule("gravity");
		this.ax = this.axG0 + (this.axG1 - this.axG0) * gravity;
		this.ay = this.ayG0 + (this.ayG1 - this.ayG0) * gravity;
		this.vxDamp = this.vxDampG0 + (this.vxDampG1 - this.vxDampG0) * gravity;
		this.vyDamp = this.vyDampG0 + (this.vyDampG1 - this.vyDampG0) * gravity;
		
		// Movement
		var didMove = false,
			across = (actionMoveLeft || actionMoveRight) && (actionMoveUp || actionMoveDown),
			mult = across ? 1/1.414 : 1;
			
			if(actionMoveLeft ) this.vx -= delta * this.ax * mult, didMove = true;
			if(actionMoveRight) this.vx += delta * this.ax * mult, didMove = true;
			if(actionMoveUp   ) this.vy -= delta * this.ay * mult, didMove = true;
			if(actionMoveDown ) this.vy += delta * this.ay * mult, didMove = true;
		
		// Physics
		this.x += this.vx;
		this.y += this.vy;
		this.r += this.vr;
		
		this.game.desirX = this.x;
		this.game.desirY = this.y;
		
		this.vx /= this.vxDamp;
		this.vy /= this.vyDamp;
		
		// Bounds
		if(this.game.scene.getRule("useBounds")){
			var radius = 330/2;
			
			var dx = this.x - radius,
				dy = this.y - radius,
				d = Math.abs(Math.hypot(dx, dy));
			
			if(d>radius-10){
				var amount = radius - d - 10,
					angle = Math.atan2(dy, dx) * (180 / Math.PI) + 90;
				
				this.vx += amount / 50 *  Math.sin(angle/180*Math.PI),
				this.vy += amount / 50 * -Math.cos(angle/180*Math.PI);
			}
		}
		
		if(this.game.scene.getRule("canShoot")){
			// Q key cycles spell index -1
			if(input.keys[81]){
				this.spellIndex -= 1;
				if(this.spellIndex === -1) this.spellIndex = this.spells.length - 1;
				input.keys[81] = false;
			}
			
			// E key cycles spell index +1
			if(input.keys[69]){
				this.spellIndex += 1;
				if(this.spellIndex === this.spells.length) this.spellIndex = 0;
				input.keys[69] = false;
			}
			
			// Left mouse click shoots
			if(actionShoot && this.spellCooldown === 0){
				this.spellCount++;
				this.spellCooldown = this.spellCooldowns[this.spellIndex];
				this.spellCooldownMax = this.spellCooldown;
				this.game.playTrack("cast");
				
				var spell = this.spells[this.spellIndex],
					spellOffsetX = 1.5,
					spellOffsetY = 4;
				
				
				if(spell===Flamethrower){
					var that = this;
					
					this.game.animations.add(function(){
						if(that.life <= 0) return;
						
						var offsetX = spellOffsetY *  Math.sin(that.r/180*Math.PI),
							offsetY = spellOffsetY * -Math.cos(that.r/180*Math.PI);
						
						offsetX += spellOffsetX *  Math.sin((that.r + 90)/180*Math.PI),
						offsetY += spellOffsetX * -Math.cos((that.r + 90)/180*Math.PI);
						
						that.game.scene.addObject(new spell(that.game, that.x + offsetX, that.y + offsetY, that.r, that.spellCount % 2 === 0 ? 1 : -1));
					}, null, 1);
				}
				
				else{
					var offsetX = spellOffsetY *  Math.sin(this.r/180*Math.PI),
						offsetY = spellOffsetY * -Math.cos(this.r/180*Math.PI);
					
					offsetX += spellOffsetX *  Math.sin((this.r + 90)/180*Math.PI),
					offsetY += spellOffsetX * -Math.cos((this.r + 90)/180*Math.PI);
					
					this.game.scene.addObject(new spell(this.game, this.x + offsetX, this.y + offsetY, this.r, this.spellCount % 2 === 0 ? 1 : -1));
				}
			}
		}
		
		// Footstep Particles
		if(this.vx !== 0 || this.vy !== 0 || this.vr !== 0){
			var world = this.game.scene.getRule("world");
			
			if(world==="tundra" && this.game.canShowParticle(10)){
				var particle = new FootstepParticle(this.game, this.game.textures.world.tundra, this.x, this.y, this.r);
				this.game.scene.addParticle(particle);
			}
			
			if(world==="space"){
				var particle = new WizardTrail(this.game, this.texture, this.map, this.x, this.y, this.r, this.vx, this.vy);
				this.game.scene.addParticle(particle);
				
				if(didMove && this.game.canShowParticle(5)){
					var particle = new WizardTrailSparkle(this.game, this.x, this.y, this.r);
					this.game.scene.addParticle(particle);
				}
			}
		}
		
		this.spellCooldown -= delta;
		if(this.spellCooldown < 0) this.spellCooldown = 0;
		
		this.shieldCooldown -= delta;
		if(this.shieldCooldown < 0) this.shieldCooldown = 0;
		
		if(this.shieldCooldown <= 0){
			this.shield += delta / this.spellCooldownMax;
			if(this.shield>this.shieldMax) this.shield = this.shieldMax;
		}
		
		if(this.life <= 0){
			var that = this,
				minDelta = 0.015;
			
			this.game.animations.add(function(i){
				that.game.updateDelta = (1 - i) * (1 - minDelta) + minDelta;
			}, null, 2);
			
			LoseScript.call(this.game, this.game);
		}
		
		return this.life;
	}
	
	
	draw(context, textures, atlas){
		var texture = this.texture,
			map = this.map;
		
		context.save();
		context.translate(this.x, this.y);
		context.rotate(this.r/180*Math.PI);
		
		context.drawImage(texture, map[0], map[1], map[2], map[3], -this.width/2, -this.height/2 - 1 * this.scale, this.width, this.height);

		context.restore();
	}
}
